/*  smplayer, GUI front-end for mplayer.
    Copyright (C) 2006-2015 Ricardo Villalba <rvm@users.sourceforge.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
*/


#ifndef _HELPER_H_
#define _HELPER_H_

#include <QString>
#include <QStringList>
#include "preferences.h"

class Helper {

public:

	// Format a time (hh:mm:ss)
	static QString formatTime(int secs);

	static QString timeForJumps(int secs);

	// Give a name for config (group name) based on dvd id
	/* static QString dvdForPref(const QString & dvd_id, int title); */

	static void msleep(int ms);

	//! Change filenames like "C:/Program Files/" to "C:\Program Files\"
	static QString changeSlashes(QString filename);

	static bool directoryContainsDVD(QString directory);

	//! Returns an int with the version number of Qt at run-time.
    //! If version is 4.3.2 it returns 40302.
	static int qtVersion();

	//! Returns a string to be passed to mplayer with the audio equalizer
	//! values.
//	static QString equalizerListToString(AudioEqualizerList values);

	static QStringList filesForPlaylist(const QString & initial_file, Preferences::AutoAddToPlaylistFilter filter);

#ifdef Q_OS_LINUX
	//! Tries to find the executable in the path.
	//! Returns the path if found or QString::null if not.
	static QString findExecutable(const QString & name);
#endif

private:
	static QStringList searchForConsecutiveFiles(const QString & initial_file);
	static QStringList filesInDirectory(const QString & initial_file, const QStringList & filter);
};

#endif
